#------------------------------------------------------------------
# $Copyright (c) 2019 Veritas Technologies LLC.  All rights reserved. Veritas
# and the Veritas Logo are trademarks or registered trademarks of Veritas
# Technologies LLC or its affiliates in the U.S. and other countries. Other
# names may be trademarks of their respective owners.
#
# The Licensed Software and Documentation are deemed to be commercial computer
# software as defined in FAR 12.212 and subject to restricted rights as defined
# in FAR Section 52.227-19 "Commercial Computer Software - Restricted Rights" and
# DFARS 227.7202, et seq. "Commercial Computer Software and Commercial Computer
# Software Documentation," as applicable, and any successor regulations, whether
# delivered by Veritas as on premises or hosted services. Any use, modification,
# reproduction release, performance, display or disclosure of the Licensed Software
# and Documentation by the U.S. Government shall be solely in accordance with the
# terms of this Agreement. $
# 
# $Header: /project/isv-agent/VProAgents/SystemD/SystemD.pm,v 1.20 2015/06/18 11:30:42 avaidya Exp $ 
#
# $Author: avaidya $
# $Date: 2015/06/18 11:30:42 $
# $Revision: 1.20 $
#
# Provides:                            SystemD
# Author:                              avaidya
# crAgent Build:                       1.88 2008/10/13 08:28:02
# crAgent Run Date:                    13 Oct, 2008
# Modification History:
# Known bugs/caveats:
#------------------------------------------------------------------

#------------
# Be strict..
#------------
use strict;
use warnings;

#----------------------
# Declare the package..
#----------------------
package SystemD;

use ag_i18n_inc;

#-------------------------------------
# Include required framework modules..
#-------------------------------------
use Sys;            # Support generalized UNIX commands
use Proc;           # For calls to the process routines
use Comms;          # For calls to socket(3C) routines
use Version;        # Version display generator
use VCSagentFW;     # Primary Agent Interface to the ACC Library
use VCSDefines;     # This contains constants that VCS uses

#--------------------------------
# Include standard PERL modules..
#--------------------------------
use Fcntl;
use Socket;
use POSIX qw ( uname );
use POSIX qw ( strftime );
use File::Path qw ( mkpath );
use File::stat;

my ( $CLUS_HOME, $sPrefix ) = ();

#------------------------
# Setup VCS/VCSone home..
#------------------------
if ( $ENV { CLUSTER_HOME } ) { # VCS 5.x/VCSOne..
  $CLUS_HOME ||= $ENV { CLUSTER_HOME } ||= "/opt/VRTSvcsone";
}
else { # VCS 4.x..
  $CLUS_HOME ||= $ENV { VCS_HOME } ||= "/opt/VRTSvcs";
}

( $sPrefix ) = $CLUS_HOME =~ /.*(vcs(|one))$/;

if ( $sPrefix =~ /vcsone/ ) {
   $sPrefix = "ha";
}

#-----------------------------------------
# Define module-vendor specific commands..
#-----------------------------------------
my $sCmdVcsDecrypt = "$CLUS_HOME/sbin/${sPrefix}decrypt";

#-----------------------------------------------------------------------
# Add our version number to the version hash and check if versions are
# requested..
#-----------------------------------------------------------------------
my $sVer = "\$Id: SystemD.pm,v 1.20 2015/06/18 11:30:42 avaidya Exp $::ETVHeader";
Version::TellVersion ( $sVer );
my $SYSTEMCTL = "/usr/bin/systemctl";
my $GREP = "/bin/grep";

#-----------------
# Define globals..
#-----------------

#--------------------------------
# Define SystemD subroutines..
#--------------------------------

=pod
 -------------------------------------------------------------------------
 Function : int DetectSystemd ();

 Description : Detect whether SystemD support in  current operating system(RHEL7/SLES12 or higher). 
 
 Parameters  :

 Returns : boolean value in the set of {0,1} where
           0 false
           1 true
 -------------------------------------------------------------------------

=cut

sub DetectSystemd {
  #----------------
  # Get Arguments..
  #----------------

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];
  
  &logTrace ( 20900, "Subroutine <%s> called .",
    $sMyName);

  #-------------------------
  # Define local variables..
  #-------------------------
  my ($cmd);
  my ($pid_one, $tty, $time, @psargs);
  my ($processname);
  my ($SYSTEM) = uname();
  my $is_systemd = 0;

  if ($SYSTEM eq "Linux") {
    $cmd = "/bin/ps -e";
    unless (open(PS, "$cmd | $GREP \" 1 ?\"|")) {
      	return 0;
    }
# output of ps command
#     1 ?        00:09:49 systemd
    while (my $line = <PS>) {
      chomp($line);
      $line =~ s/^\s*(.*?)\s*$/$1/;
      ($pid_one, $tty, $time, @psargs) = split(/\s+/, $line);
      ($processname) = join(' ', @psargs);

      if ( ( 1 == $pid_one ) && ($processname =~ /^systemd\b/) ) {
        $is_systemd = 1;
        last;
      }
    }
       
    close(PS);
  } #if ($SYSTEM eq "Linux")
  return $is_systemd;

}

=pod
 -------------------------------------------------------------------------
 Function : int CreateSystemd ();

 Description : Creates systemd service file for each resource . 
 
 Parameters  :
 ResName           Resource name 
 User              user of the command for which service needs to be created
 Command           Command to be specified in the service file
 CommandArgs       Arguments of the command to be specified in the service file
 EnvFile           Enivironment File 
 TimeOutSec        timeout  needs to be passed to service file
 sCommandName      Command for which service needs to be created
 SuccessExitStatus exit status definitions that will be considered successful termination

 Returns : boolean value in the set of {0,1} where
           0 false
           1 true
 -------------------------------------------------------------------------

=cut

sub CreateSystemd {
  #----------------
  # Get Arguments..
  #----------------

  my($ResName, $User, $sCommand, $sCommandArgs, $sEnvFile, $TimeOutSec, $sCommandName, $SuccessExitStatus) = @_;
  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];
  &logTrace ( 20901, "Subroutine <%s> called with:
  ResName           [%s]
  User              [%s]
  Command           [%s]
  CommandArgs       [%s]
  EnvFile           [%s]
  TimeOutSec        [%s]
  sCommandName      [%s]
  SuccessExitStatus [%s]",
  $sMyName, $ResName, $User, $sCommand, $sCommandArgs, $sEnvFile, $TimeOutSec, $sCommandName, $SuccessExitStatus);

  #-------------------------
  # Define local variables..
  #-------------------------
  my $service_file = "/etc/systemd/system/vcs-$ResName.service";
  if ( defined $sCommandName ) {
    $service_file = "/etc/systemd/system/vcs-$ResName-$sCommandName.service";
  }
  my $ret_val = 0; #failure- change
  my $sResourceType = ( $0 =~ m@([^/]+)@g ) [-2];
  my $procType = "forking";
  if(defined $sCommandName && $sCommandName eq "simple"){
	  $procType = $sCommandName;
  }

# SuccessExitStatus=
# Takes a list of exit status definitions that, when returned by the main service process, 
# will be considered successful termination, in addition to the normal successful exit code 0 
# and the signals SIGHUP, SIGINT, SIGTERM, and SIGPIPE. 
# Exit status definitions can either be numeric exit codes or 
# termination signal names, separated by spaces. For example:
#
# SuccessExitStatus=1 2 8 SIGKILL
# ensures that exit codes 1, 2, 8 and the termination signal SIGKILL are considered clean service terminations.
#
# This option may appear more than once, in which case the list 
# of successful exit statuses is merged. 
# If the empty string is assigned to this option, the list is reset, 
# all prior assignments of this option will have no effect.

  if(!defined $SuccessExitStatus) {
    $SuccessExitStatus = "";
  }

  if ( sysopen ( FILE, $service_file, O_WRONLY|O_SYNC|O_CREAT|O_TRUNC, 0770 ) ) {

    printf FILE "#$service_file

[Unit]
Description=Veritas $sResourceType service file

[Service]
Type=$procType
Restart=no
KillMode=none
SendSIGKILL=no
EnvironmentFile=$sEnvFile
ExecStart=$sCommand $sCommandArgs
TimeoutStartSec=$TimeOutSec
User=$User
SuccessExitStatus=$SuccessExitStatus

[Install]
WantedBy=multi-user.target";

	$ret_val = 1;
  }
  else
  {
    &logError ( 20902, "<%s> Failed to create the systemd service file [%s] ", 
    $sMyName,$service_file);
    $ret_val = 0;
  }
  close(FILE);
  return $ret_val;

}
=pod
 -------------------------------------------------------------------------
 Function : int RunSystemd ();

 Description : Runs systemd service file for each resource . 
 
 Parameters  :
 ResName       Resource name 
 sCommandName  Command for which service needs to be created

 Returns : boolean value in the set of {0,1} where
           0 FAILURE
           1 SUCCESS
 -------------------------------------------------------------------------

=cut

sub RunSystemd {
  #----------------
  # Get Arguments..
  #----------------

  my ( $ResName , $sCommandName) = @_;
  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];
  
  &logTrace ( 20903, "Subroutine <%s> called with:
  ResName         [%s]
  CommandName     [%s].",
  $sMyName, $ResName, $sCommandName);

  #-------------------------
  # Define local variables..
  #-------------------------
  my $Command = "$SYSTEMCTL daemon-reload";
  &logTrace ( 20904, "<%s> Executing command [%s].",$sMyName,$Command);
  my $iExitCode = system($Command) ;
  my $iReturnCode = 1;
  my $serviceName = "vcs-$ResName";
  if ( defined $sCommandName ) {
    $serviceName = "vcs-$ResName-$sCommandName";
  }

  if ( $iExitCode == 0 ) {
    &logTrace ( 20905, "<%s> Cmd [%s] executed successfully",$sMyName, $Command);

    $Command = "$SYSTEMCTL disable $serviceName";
    &logTrace ( 20906, "<%s> Executing command [%s].",$sMyName,$Command);
    $iExitCode = system($Command) ;

    if ( $iExitCode == 0 ) {
      &logTrace ( 20907, "<%s> Cmd [%s] executed successfully",$sMyName, $Command);

      $Command = "$SYSTEMCTL start $serviceName";
      &logTrace ( 20908, "<%s> Executing command [%s].",$sMyName,$Command);
      $iExitCode = system($Command) ;

      if ( $iExitCode != 0 ) {
        &logError ( 20909, "<%s> Cmd [%s] is not executed successfully", $sMyName,$Command);
        $iReturnCode = 0;
      } else {
        &logTrace ( 20912, "<%s> Cmd [%s] executed successfully",$sMyName, $Command);
      }

      $Command = "$SYSTEMCTL -l status $serviceName";
      &logTrace ( 20910, "<%s> Executing command [%s].",$sMyName,$Command);

      my $outMessage = `$SYSTEMCTL -l status $serviceName 2>&1`;
      &logTrace ( 20911, "<%s> outMessage is [%s]",$sMyName, $outMessage);
    } else {
      &logError ( 20913, "<%s> Cmd [%s] is not executed successfully",$sMyName, $Command);
      $iReturnCode = 0;
    }
 
  } else {
    &logError ( 20914, "<%s> Cmd [%s] is not executed successfully",$sMyName, $Command);
    $iReturnCode = 0;
  }

  return $iReturnCode;
}

=pod
 -------------------------------------------------------------------------
 Function : int RunSystemdWithEnv ();

 Description : Creates and Runs systemd service file for each resource . 
 
 Parameters  :
 User              user of the command for which service needs to be created
 Command           Command to be specified in the service file
 CommandArgs       Arguments of the command to be specified in the service file
 EnvFile           EnvFile which needs to be passed to service file
 TimeOutSec        timeout  needs to be passed to service file
 sCommandName      Command for which service needs to be created
 SuccessExitStatus exit status definitions that will be considered successful termination


 Returns : boolean value in the set of {0,1} where
           0 false
           1 true
 -------------------------------------------------------------------------

=cut

sub RunSystemdWithEnv {
  #----------------
  # Get Arguments..
  #----------------

  my($User, $sCommand, $sCommandArgs, $sEnvFile, $iTimeOutSec, $sCommandName, $SuccessExitStatus) = @_;
  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];
  &logTrace ( 20915, "Subroutine <%s> called with:
  User              [%s]
  Command           [%s]
  CommandArgs       [%s]
  EnvFile           [%s]
  TimeOutSec        [%s]
  CommandName       [%s]
  SuccessExitStatus [%s]",
  $sMyName, $User, $sCommand, $sCommandArgs, $sEnvFile, $iTimeOutSec, $sCommandName, $SuccessExitStatus);

  #-------------------------
  # Define local variables..
  #-------------------------
  &logTrace ( 20916,  "<%s> SYSTEMD is detected",$sMyName);
  #--------------------------------------
  # Set Return Code Optimistically to 1..
  #--------------------------------------
  my $iReturnCode = 1; 

  my $ResName = $ENV { VCS_LOG_RESOURCE_NAME };
  $iReturnCode = CreateSystemd($ResName,$User,$sCommand,$sCommandArgs,$sEnvFile, $iTimeOutSec, $sCommandName, $SuccessExitStatus);
  if($iReturnCode) {
    &logTrace ( 20917,  "<%s> CreateSystemd was successful",$sMyName);
    $iReturnCode = RunSystemd($ResName,$sCommandName);
    if($iReturnCode) {
      &logTrace ( 20918, "<%s> RunSystemd was successful",$sMyName);
    } else {
      &logError ( 20919, "<%s> RunSystemd was not successful",$sMyName);
    }
  } else {
    &logError ( 20920, "<%s> CreateSystemd was not successful",$sMyName);
  }
  
  return $iReturnCode;
}


=pod
 -------------------------------------------------------------------------
 Function : int RunSystemdServiceName ();

 Description : Runs systemd service file for each resource 
               when the Service Name is specified. 
 
 Parameters  :
 ResName       Resource name 
 ServiceName  Name of the service to be started
 CommandArg   Argument to the systemctl command eg. start, stop, is-active

 Returns : boolean value in the set of {0,1} where
           0 FAILURE
           1 SUCCESS
 -------------------------------------------------------------------------

=cut

sub RunSystemdServiceName {
  #----------------
  # Get Arguments..
  #----------------

  my ( $sResName , $sServiceName, $sCommandArg) = @_;
  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];
  
  &logTrace ( 20921, "Subroutine <%s> called with:
  ResName         [%s]
  ServiceName     [%s]
  CommandArg      [%s]",
  $sMyName, $sResName, $sServiceName, $sCommandArg);

  #-------------------------
  # Define local variables..
  #-------------------------
  if ( ! defined $sServiceName ) {
     &logError ( 20922, "<%s>: Service Name not defined.", $sMyName );    
     return 0;
  }
  
  if ( ! defined $sCommandArg ) {
     &logError ( 20923, "<%s>: Command Arguments not defined.", $sMyName );
     return 0;
  }

  my $sCommand = "$SYSTEMCTL daemon-reload";
  &logTrace ( 20924, "<%s> Executing command [%s].",$sMyName,$sCommand);
  my $iExitCode = system($sCommand) ;
  my $iReturnCode = 1;
  if ( $iExitCode == 0 ) {
    &logTrace ( 20925, "<%s> Cmd [%s] executed successfully",$sMyName, $sCommand);

    $sCommand = "$SYSTEMCTL disable $sServiceName";
    &logTrace ( 20926, "<%s> Executing command [%s].",$sMyName,$sCommand);
    $iExitCode = system($sCommand) ;

    if ( $iExitCode == 0 ) {
      &logTrace ( 20927, "<%s> Cmd [%s] executed successfully",$sMyName, $sCommand);

      $sCommand = "$SYSTEMCTL $sCommandArg $sServiceName";
      &logTrace ( 20928, "<%s> Executing command [%s].",$sMyName,$sCommand);
      $iExitCode = system($sCommand) ;

      if ( $iExitCode != 0 ) {
        &logError ( 20929, "<%s> Cmd [%s] is not executed successfully", $sMyName,$sCommand);
        $iReturnCode = 0;
      } else {
        &logTrace ( 20930, "<%s> Cmd [%s] executed successfully",$sMyName, $sCommand);
      }

      $sCommand = "$SYSTEMCTL -l status $sServiceName";
      &logTrace ( 20931, "<%s> Executing command [%s].",$sMyName,$sCommand);

      my $outMessage = `$SYSTEMCTL -l status $sServiceName 2>&1`;
      &logTrace ( 20932, "<%s> outMessage is [%s]",$sMyName, $outMessage);
    } else {
      &logError ( 20933, "<%s> Cmd [%s] is not executed successfully",$sMyName, $sCommand);
      $iReturnCode = 0;
    }
 
  } else {
    &logError ( 20934, "<%s> Cmd [%s] is not executed successfully",$sMyName, $sCommand);
    $iReturnCode = 0;
  }

  return $iReturnCode;
}

